/*
 * Project name:
     ADC Demo
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
      A simple example of using the ADC library.
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       Clicker 2 for FT900
                      http://www.mikroe.com/ft90x/clicker-2/
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    LPG click  - ac:LPG_Click
                      UART click - ac:UART_Click
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place LPG click in the mikroBUS socket 2
     - Place UART click in the mikroBUS socket 1
     - Read the values from LPG click over USART terminal
 */

#include <stdint.h>

// undefine the option you would like to use
#define ADC_GET_SAMPLE
// #define ADC_READ

typedef enum LineEnding {WIN, UNIX} LineEnding;

uint16_t
   ADC_Result = 0,
   baudRate   = 56000;

LineEnding
  linEnd = WIN;

uint8_t
  tempStr[12];

/**
 * Sends the line along with its terminating characters
 * @param text     pointer to string to be sent
 * @param linEnd   choose the line ending either for WIN or UNIX
 */
void UART1_Write_Line(uint8_t* text,
                    LineEnding linEnd)
{
  if (linEnd == WIN)
  {
    UART1_Write_Text(text);
    UART1_Write(13);
    UART1_Write(10);
  }
  else
  {
    UART1_Write_Text(text);
    UART1_Write(10);
  }
}

/**
 * Reads value from LPG Click
 */
void ReadSensor()
{
  #ifdef ADC_GET_SAMPLE
    ADC_Result = ADC_Get_Sample(1);
  #else
    ADC_Result = ADC_Read(1);                // Read RA0
  #endif
  Delay_ms(10);                              // Pause 10ms
}

/**
 * Read the values from the sensor in an infinite loop
 */
void main()
{
  // Initialize UART and send START msg
  UART1_Init      (baudRate          );
  UART1_Write_Line("ADC Demo", linEnd);
  UART1_Write_Line("Start",    linEnd);

  #ifdef ADC_GET_SAMPLE
    // Initialize ADC
    ADC_Init();
    ADC_Set_Input_Channel(_ADC_CHANNEL_1);
  #endif

  while(1)
  {
    // Get ADC value
    ReadSensor();

    // Send the value as a string via UART
    WordToStr       (ADC_Result, tempStr);
    UART1_Write_Text("Read value: "     );
    UART1_Write_Line(tempStr,     linEnd);

    Delay_ms(500);
  }
}