/*
 * Project name:
     ManchesterReceiver (Demonstration on usage of Manchester code library functions)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
     This code shows how to use Manchester Library for receiving data. The
     example works in conjuction with Transmitter example which sends the word
     "mikroElektronika" using Manchester encoding. During the receive process,
     message letters are being written on the UART.
 * Test configuration:
     MCU:             FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       Clicker 2 for FT900
                      http://www.mikroe.com/ft90x/clicker-2/
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. modules:    USB UART click ac:USB_UART_Click
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place UAB UART click on mikroBUS 2 socket and connect it to PC via USB cable.
     - Connect Rx, Tx and GND pins via wire jumper to transmitter clicker.
     - Open UART terminal and watch receive process.
 */

// Manchester module connections
sbit MANRXPIN at GPIO_PIN12_bit;
sbit MANTXPIN at GPIO_PIN11_bit;
// End Manchester module connections

unsigned int error;
char ErrorCount, chr_counter, byte_rcvd;

void main() {

  ErrorCount = 0;
  chr_counter = 0;

  UART1_Init(19200);                              // Initialize UART

  Man_Receive_Init();                             // Initialize Receiver

  while (1) {                                     // Endless loop
      UART_Write_Text("\r\n");                    // Print new line
      while (1) {                                 // Wait for the "start" byte
        byte_rcvd = Man_Receive(&error);          // Attempt byte receive
        if (byte_rcvd == 0x0B)                    // "Start" byte, see Transmitter example
          break;                                  // We got the starting sequence
        if (error)                                // Exit so we do not loop forever
          break;
      }

      do
        {
          byte_rcvd = Man_Receive(&error);        // Attempt byte receive
          if (error) {                            // If error occured
            UART_Write('?');                      // Write question mark on UART
            ErrorCount++;                         // Update error counter
            if (ErrorCount > 20) {                // In case of multiple errors
              Man_Synchro();                      // Try to synchronize again
              //Man_Receive_Init();               // Alternative, try to Initialize Receiver again
              ErrorCount = 0;                     // Reset error counter
              }
          }

          else {                                  // No error occured
            if (byte_rcvd != 0x0E) {              // If "End" byte was received(see Transmitter example)
                                                  // do not write anymore received byte on UART
              UART_Write(byte_rcvd);              // else write character on UART
              chr_counter++;                      // Counts how many chars have been written on UART
              if (chr_counter == 25) {            // If there were more then 25 characters
                                                  // synchronization is off
                UART_Write_Text("\r\n");          // Print new line
                Man_Synchro();                    // Try to synchronize again
              }
            }
            else
              chr_counter = 0;                    // reset chr_counter
          }

          Delay_ms(25);
        }
      while (byte_rcvd != 0x0E);                  // If "End" byte was received exit do loop
   }
}