/*
 * Project name:
     Button (Demonstration of using Button Library)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
     This program demonstrates usage on-board button as GPIO_PORT_08_15 input.
     On every GPIO_PIN8 one-to-zero transition GPIO_PORT_16_23 is inverted.
 * Test configuration:
     MCU:             FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev. Board:      EasyFT90x v7
                      http://www.mikroe.com/easyft90x
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. modules:    None.
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Turn ON the GPIO_PORT_16_23 LEDs at SW8.3.
     - Pull-up switch SW1.2 (VCC position) and
     - Pull-down GPIO_PORT_08_15.B0 at SW21.8.
 */

unsigned int oldstate;

void main() {
  GPIO_Digital_Input(&GPIO_PORT_08_15, _GPIO_PINMASK_0);       // Set GPIO_PIN8 as digital input
  GPIO_Digital_Output(&GPIO_PORT_16_23, _GPIO_PINMASK_ALL);    // Set GPIO_PORT_16_23 as digital output

  GPIO_PORT_16_23 = 0x0F;                                      // Initial GPIO_PORT_16_23 value
  oldstate = 0;

  do {
    if (Button(&GPIO_PORT_08_15, 0, 1, 1))                    // detect logical one on GPIO_PIN8 pin
      oldstate = 1;
    if (oldstate && Button(&GPIO_PORT_08_15, 0, 1, 0)) {      // detect one-to-zero transition on GPIO_PIN8 pin
      GPIO_PORT_16_23 = ~GPIO_PORT_16_23;                     // Invert GPIO_PORT_16_23
      oldstate = 0;                                           // Update flag
    }
  } while(1);                                                 // endless loop
}