/* Project name:
     CAN_1st (can network demonstration)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release;
 * Description:
     This code demonstrates how to use CAN library functions and procedures.
     It is used together with the CAN_2nd example (on the second MCU), and it can
     be used to test the connection of an MCU to the CAN network.
     This node initiates the communication with the 2nd node by sending some
     data to its address. The 2nd node responds by sending back data incre-
     mented by 1. This (1st) node then does the same and sends incremented data
     back to 2nd node, etc.
     With minor adjustments, it should work with any other MCU that has a CAN module.
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       EasyFT90x v7 - ac:CAN
                      http://www.mikroe.com/easyft90x/

     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Connect USB cable and turn ON USB UART switches at SW2.2 and SW2.3 (board specific).
     - Turn on GPIO PORT 24-31 LEDs at SW8 (board specific).
     - Turn on CAN RX and TX switches at SW3.3 and SW3.4 (board specific).
     - Observe the exchange of data at GPIO PORT 24-31 and over USART terminal.
 */

#include "stdint.h"
#include "__can.h"
#include "__Can_Hw_Map.h"

uint8_t
  uart_rd,
  txt[10];

TCANMsg
  msg,
  rcv;

uint32_t
  flags;

void main()
{
  // Initialize UART module at 56000 bps
  UART1_Init(56000);
  // Wait for UART module to stabilize
  Delay_ms(100);

  UART1_Write_Text("Start of the transmitter");
  UART1_Write(13);
  UART1_Write(10);

  // HW setup of CAN1 module at GPIO15 and GPIO16
  CAN_SysInit(CAN1);

  if ( CAN_ERROR == CAN_Init(CAN1, 250) )
  {
    // initiate CAN0 module at 250kB bitrate
    UART1_Write_Text("CAN_Init error");
  }

  if ( CAN_ERROR == CAN_SetFilter(CAN1, 112 , DEFAULT) )
  {
    UART1_Write_Text("could not set the filter");
  }

  msg.RTR    = 0;
  msg.ID     = 13;
  msg.len    = 1;
  msg.buf[0] = 0xaa;

  CAN_PushMessage(CAN1, &msg);
  Delay_ms(1000);

  while(1)
  {
    uint8_t
    count;
    count = CAN_RXQueueCount(CAN1);
    if (count)
    {
      CAN_PullMessage(CAN1, &msg);
      if (msg.len)
      {
        IntToStr(msg.buf[0], txt);
        UART1_Write_Text(txt);
        UART1_Write(13);
        UART1_Write(10);
        Delay_ms(500);

        msg.RTR = 0;
        msg.ID  = 13;
        msg.len = 1;
        flags   = 0;

        msg.buf[0]++;
        CANx_Write(CAN1, msg.ID, msg.buf, msg.len, flags);
      }
    }
  }
}