/*
** can_hw_map.h
**
** Copyright © 2013 Future Technology Devices International Limited
**
** Company Confidential
**
** Header file for FT900 CAN register map
**
** Author: Huynh Le Trung
** Project: FT900
** Module: FT900 CAN driver
** Requires:
** Comments:
**
** History:
**  1 - initial version
*/

#ifndef __FT900_CAN_MAP_H_
#define __FT900_CAN_MAP_H_

#include <stdint.h>

// ============================================================================================== //
//                                 P E R I P H E R A L   R E G I S T E R S                                                               //
// ============================================================================================== //
#define CAN1_BASE                        (V3_IO_OFFSET + 0x240)
#define CAN2_BASE                        (V3_IO_OFFSET + 0x260)

// As verification found out, byte access can screw up TX and RX FIFO, so use 32-bit access here
#define DWORD_ACCESS

typedef struct
{
        int8_t CAN_MR;                  // mode register
        int8_t CAN_CMD;                 // command register
        int8_t CAN_SR;                  // status register
        int8_t CAN_ISR_IACK;            // interrupt status / acknowledge register
        int8_t CAN_IMR;                 // interrupt mask register
        int8_t CAN_RMC;                 // receive message counter register
        int8_t CAN_BTR0;                // bus timing register 0
        int8_t CAN_BTR1;                // bus timing register 1
#if defined(BYTE_ACCESS)
        int8_t CAN_TXBUF0;              // transmit buffer register 0
        int8_t CAN_TXBUF1;              // transmit buffer register 1
        int8_t CAN_TXBUF2;              // transmit buffer register 2
        int8_t CAN_TXBUF3;              // transmit buffer register 3
        int8_t CAN_RXBUF0;              // receive buffer register 0
        int8_t CAN_RXBUF1;              // receive buffer register 1
        int8_t CAN_RXBUF2;              // receive buffer register 2
        int8_t CAN_RXBUF3;              // receive buffer register 3
#elif defined(DWORD_ACCESS)
        int32_t CAN_TXBUF;              // transmit buffer register (32-bit access)
        int32_t CAN_RXBUF;              // receive buffer register (32-bit access)
#endif
        int8_t CAN_ACR0;                // acceptance code register 0
        int8_t CAN_ACR1;                // acceptance code register 1
        int8_t CAN_ACR2;                // acceptance code register 2
        int8_t CAN_ACR3;                // acceptance code register 3
        int8_t CAN_AMR0;                // acceptance mask register 0
        int8_t CAN_AMR1;                // acceptance mask register 1
        int8_t CAN_AMR2;                // acceptance mask register 2
        int8_t CAN_AMR3;                // acceptance mask register 3
        int8_t CAN_ECC;                 // error code capture register
        int8_t CAN_RXERR;               // RX error counter register
        int8_t CAN_TXERR;               // TX error counter register
        int8_t CAN_ALC;                 // management address register
} TCANStruct;

// ============================================================================================== //
//                                        P E R I P H E R A L   L I S T                           //
// ============================================================================================== //
#define CAN1 ((TCANStruct *)&CAN1_MODE)
#define CAN2 ((TCANStruct *)&CAN2_MODE)

#endif // __FT900_CAN_MAP_H_