#include "Camera_Demo_objects.h"
#include "Camera_Demo_resources.h"
#include "built_in.h"

// Extern memory interface
sbit Mmc_Chip_Select           at GPIO_PIN0_bit;
char Ext_Data_Buffer[512];
unsigned long currentSector = -1, res_file_size;
extern char *GetCameraFrame(unsigned long bytesToGet, unsigned long *pBytesGot);
// End of extern memory interface


// TFT module connections
char TFT_DataPort at GPIO_PORT_32_39;
sbit TFT_RST at GPIO_PIN42_bit;
sbit TFT_RS at GPIO_PIN40_bit;
sbit TFT_CS at GPIO_PIN41_bit;
sbit TFT_RD at GPIO_PIN46_bit;
sbit TFT_WR at GPIO_PIN47_bit;
sbit TFT_BLED at GPIO_PIN58_bit;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at GPIO_PIN0_bit;
sbit DriveX_Right at GPIO_PIN0_bit;
sbit DriveY_Up at GPIO_PIN0_bit;
sbit DriveY_Down at GPIO_PIN0_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 750;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton *local_button;
TButton *exec_button;
int button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TImage *local_image;
TImage *exec_image;
int image_order;
TBox *local_box;
TBox *exec_box;
int box_order;


char* TFT_Get_Data(unsigned long offset, unsigned long count, unsigned long *num) {
  unsigned long start_sector;
  unsigned int pos;

  if(offset >= imageQVGA_bmp)
  {
    return GetCameraFrame(count, num);
  }

  start_sector = Mmc_Get_File_Write_Sector() + offset/512;
  pos = (unsigned long)offset%512;

  if(start_sector == currentSector+1) {
    Mmc_Multi_Read_Sector(Ext_Data_Buffer);
    currentSector = start_sector;
  } else if (start_sector != currentSector) {
    if(currentSector != -1)
      Mmc_Multi_Read_Stop();
    Mmc_Multi_Read_Start(start_sector);
    Mmc_Multi_Read_Sector(Ext_Data_Buffer);
    currentSector = start_sector;
  }

  if(count>512-pos)
    *num = 512-pos;
  else
    *num = count;

  return Ext_Data_Buffer+pos;
}

void Auxiliary_Code() {
  // Place your code here
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index(unsigned short index) {
  // Place your code here
  TFT_RS = 0;
  asm nop;
  TFT_DataPort = index;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Command(unsigned short cmd) {
  // Place your code here
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = cmd;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Data(unsigned int _data) {
  // Place your code here
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = Hi(_data);
  asm nop; asm nop;
  Auxiliary_Code();
  asm nop;
  TFT_DataPort = Lo(_data);
  asm nop; asm nop;
  Auxiliary_Code();
}

static void InitializeTouchPanel() {
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_Set_Ext_Buffer(TFT_Get_Data);

}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TBox                   Box1;
  TLabel                 Label_ProjName;
char Label_ProjName_Caption[12] = "Camera Demo";

  TButton               Button_Message;
char Button_Message_Caption[41] = "Welcome";

  TLine                  Line_Up;
  TLine                  Line_Down;
  TImage               Image4;
  TImage               Image5;
  TBox                   Box3;
  TBox                   Box4;
  TButton                * const code Screen1_Buttons[1]=
         {
         &Button_Message       
         };
  TLabel                 * const code Screen1_Labels[1]=
         {
         &Label_ProjName       
         };
  TImage                 * const code Screen1_Images[2]=
         {
         &Image4,              
         &Image5               
         };
  TBox                   * const code Screen1_Boxes[3]=
         {
         &Box1,                
         &Box3,                
         &Box4                 
         };
  TLine                  * const code Screen1_Lines[2]=
         {
         &Line_Up,             
         &Line_Down            
         };

  TScreen                Screen_Camera;
  TImage               Image1;
  TImage                 * const code Screen2_Images[1]=
         {
         &Image1               
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xFFFF;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.ButtonsCount              = 1;
  Screen1.Buttons                   = Screen1_Buttons;
  Screen1.LabelsCount               = 1;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.ImagesCount               = 2;
  Screen1.Images                    = Screen1_Images;
  Screen1.BoxesCount                = 3;
  Screen1.Boxes                     = Screen1_Boxes;
  Screen1.LinesCount                = 2;
  Screen1.Lines                     = Screen1_Lines;
  Screen1.ObjectsCount              = 9;

  Screen_Camera.Color                     = 0x5AEB;
  Screen_Camera.Width                     = 320;
  Screen_Camera.Height                    = 240;
  Screen_Camera.ButtonsCount              = 0;
  Screen_Camera.LabelsCount               = 0;
  Screen_Camera.ImagesCount               = 1;
  Screen_Camera.Images                    = Screen2_Images;
  Screen_Camera.BoxesCount                = 0;
  Screen_Camera.LinesCount                = 0;
  Screen_Camera.ObjectsCount              = 1;


  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 0;
  Box1.Left            = 2;
  Box1.Top             = 2;
  Box1.Width           = 316;
  Box1.Height          = 236;
  Box1.Pen_Width       = 5;
  Box1.Pen_Color       = 0xC618;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 0;
  Box1.Gradient_Orientation = 0;
  Box1.Gradient_Start_Color = 0xFFFF;
  Box1.Gradient_End_Color = 0xC618;
  Box1.Color           = 0xFFFF;
  Box1.PressColEnabled = 1;
  Box1.Press_Color     = 0xE71C;
  Box1.OnUpPtr         = 0;
  Box1.OnDownPtr       = 0;
  Box1.OnClickPtr      = 0;
  Box1.OnPressPtr      = 0;

  Label_ProjName.OwnerScreen     = &Screen1;
  Label_ProjName.Order           = 1;
  Label_ProjName.Left            = 56;
  Label_ProjName.Top             = 44;
  Label_ProjName.Width           = 226;
  Label_ProjName.Height          = 47;
  Label_ProjName.Visible         = 1;
  Label_ProjName.Active          = 1;
  Label_ProjName.Caption         = Label_ProjName_Caption;
  Label_ProjName.FontName        = Tahoma34x42_Regular;
  Label_ProjName.Font_Color      = 0x0000;
  Label_ProjName.OnUpPtr         = 0;
  Label_ProjName.OnDownPtr       = 0;
  Label_ProjName.OnClickPtr      = 0;
  Label_ProjName.OnPressPtr      = 0;

  Button_Message.OwnerScreen     = &Screen1;
  Button_Message.Order           = 2;
  Button_Message.Left            = 8;
  Button_Message.Top             = 95;
  Button_Message.Width           = 296;
  Button_Message.Height          = 27;
  Button_Message.Pen_Width       = 1;
  Button_Message.Pen_Color       = 0xFFFF;
  Button_Message.Visible         = 1;
  Button_Message.Active          = 1;
  Button_Message.Transparent     = 1;
  Button_Message.Caption         = Button_Message_Caption;
  Button_Message.TextAlign       = _taCenter;
  Button_Message.FontName        = Tahoma17x21_Regular;
  Button_Message.PressColEnabled = 1;
  Button_Message.Font_Color      = 0x0000;
  Button_Message.Gradient        = 0;
  Button_Message.Gradient_Orientation = 0;
  Button_Message.Gradient_Start_Color = 0x0000;
  Button_Message.Gradient_End_Color = 0xC618;
  Button_Message.Color           = 0xFFFF;
  Button_Message.Press_Color     = 0xFFFF;
  Button_Message.OnUpPtr         = 0;
  Button_Message.OnDownPtr       = 0;
  Button_Message.OnClickPtr      = 0;
  Button_Message.OnPressPtr      = 0;

  Line_Up.OwnerScreen     = &Screen1;
  Line_Up.Order           = 3;
  Line_Up.First_Point_X   = 0;
  Line_Up.First_Point_Y   = 0;
  Line_Up.Second_Point_X  = 320;
  Line_Up.Second_Point_Y  = 0;
  Line_Up.Visible         = 0;
  Line_Up.Pen_Width       = 1;
  Line_Up.Color           = 0x0000;

  Line_Down.OwnerScreen     = &Screen1;
  Line_Down.Order           = 4;
  Line_Down.First_Point_X   = 0;
  Line_Down.First_Point_Y   = 240;
  Line_Down.Second_Point_X  = 320;
  Line_Down.Second_Point_Y  = 240;
  Line_Down.Visible         = 0;
  Line_Down.Pen_Width       = 1;
  Line_Down.Color           = 0x0000;

  Image4.OwnerScreen     = &Screen1;
  Image4.Order           = 5;
  Image4.Left            = 188;
  Image4.Top             = 165;
  Image4.Width           = 41;
  Image4.Height          = 40;
  Image4.Picture_Type    = 0;
  Image4.Picture_Ratio   = 1;
  Image4.Picture_Name    = mikroe_bmp;
  Image4.Visible         = 1;
  Image4.Active          = 1;
  Image4.OnUpPtr         = 0;
  Image4.OnDownPtr       = 0;
  Image4.OnClickPtr      = 0;
  Image4.OnPressPtr      = 0;

  Image5.OwnerScreen     = &Screen1;
  Image5.Order           = 6;
  Image5.Left            = 78;
  Image5.Top             = 165;
  Image5.Width           = 82;
  Image5.Height          = 40;
  Image5.Picture_Type    = 0;
  Image5.Picture_Ratio   = 1;
  Image5.Picture_Name    = ftdi_bmp;
  Image5.Visible         = 1;
  Image5.Active          = 1;
  Image5.OnUpPtr         = 0;
  Image5.OnDownPtr       = 0;
  Image5.OnClickPtr      = 0;
  Image5.OnPressPtr      = 0;

  Box3.OwnerScreen     = &Screen1;
  Box3.Order           = 7;
  Box3.Left            = 50;
  Box3.Top             = 216;
  Box3.Width           = 114;
  Box3.Height          = 5;
  Box3.Pen_Width       = 0;
  Box3.Pen_Color       = 0x0000;
  Box3.Visible         = 1;
  Box3.Active          = 1;
  Box3.Transparent     = 1;
  Box3.Gradient        = 1;
  Box3.Gradient_Orientation = 1;
  Box3.Gradient_Start_Color = 0xFFFF;
  Box3.Gradient_End_Color = 0xFDBB;
  Box3.Color           = 0xC618;
  Box3.PressColEnabled = 1;
  Box3.Press_Color     = 0xE71C;
  Box3.OnUpPtr         = 0;
  Box3.OnDownPtr       = 0;
  Box3.OnClickPtr      = 0;
  Box3.OnPressPtr      = 0;

  Box4.OwnerScreen     = &Screen1;
  Box4.Order           = 8;
  Box4.Left            = 163;
  Box4.Top             = 216;
  Box4.Width           = 114;
  Box4.Height          = 5;
  Box4.Pen_Width       = 0;
  Box4.Pen_Color       = 0x0000;
  Box4.Visible         = 1;
  Box4.Active          = 1;
  Box4.Transparent     = 1;
  Box4.Gradient        = 1;
  Box4.Gradient_Orientation = 1;
  Box4.Gradient_Start_Color = 0xFDBB;
  Box4.Gradient_End_Color = 0xFFFF;
  Box4.Color           = 0xC618;
  Box4.PressColEnabled = 1;
  Box4.Press_Color     = 0xE71C;
  Box4.OnUpPtr         = 0;
  Box4.OnDownPtr       = 0;
  Box4.OnClickPtr      = 0;
  Box4.OnPressPtr      = 0;

  Image1.OwnerScreen     = &Screen_Camera;
  Image1.Order           = 0;
  Image1.Left            = 0;
  Image1.Top             = 0;
  Image1.Width           = 320;
  Image1.Height          = 240;
  Image1.Picture_Type    = 0;
  Image1.Picture_Ratio   = 1;
  Image1.Picture_Name    = imageQVGA_bmp;
  Image1.Visible         = 1;
  Image1.Active          = 1;
  Image1.OnUpPtr         = 0;
  Image1.OnDownPtr       = 0;
  Image1.OnClickPtr      = 0;
  Image1.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetImage(index)               CurrentScreen->Images[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]
#define GetLine(index)                CurrentScreen->Lines[index]


void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Ext_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + 4, (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taCenter)
      TFT_Write_Text(Abutton->Caption, (Abutton->Left + (Abutton->Width - caption_length) / 2), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taRight)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + (Abutton->Width - caption_length - 4), (Abutton->Top + (Abutton->Height - caption_height) / 2));
  }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Ext_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawImage(TImage *AImage) {
  if (AImage->Visible) {
    TFT_Ext_Image(AImage->Left, AImage->Top, AImage->Picture_Name, AImage->Picture_Ratio);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    TFT_Set_Pen(Aline->Color, Aline->Pen_Width);
    TFT_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short box_idx;
  TBox *local_box;
  unsigned short line_idx;
  TLine *local_line;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  button_idx = 0;
  label_idx = 0;
  image_idx = 0;
  box_idx = 0;
  line_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init_ILI9341_8bit(CurrentScreen->Width, CurrentScreen->Height);
    TFT_Set_Ext_Buffer(TFT_Get_Data);
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (line_idx < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

    if (image_idx < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

void Init_MCU() {
  // Place your code here
TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Init_Ext_Mem() {
  // Init SDHost interface at low speed
  Mmc_Set_Interface(_MMC_INTERFACE_SDHOST);
  SDHost_Init(128, _SDHOST_CFG_4_WIDE_BUS_MODE | _SDHOST_CFG_CLOCK_FALING_EDGE);
  Delay_ms(100);

  // Init FAT32 stack
  if (Mmc_Fat_Init() == 0) {
    //  Reinitialize SDHost at higher speed
    SDHost_Init(8, _SDHOST_CFG_4_WIDE_BUS_MODE | _SDHOST_CFG_CLOCK_FALING_EDGE);

    // Open resource file for read
    Mmc_Fat_Assign("Camera_D.RES", 0);
    Mmc_Fat_Reset(&res_file_size);
  }
}

void Start_TP() {
  Init_MCU();

  Init_Ext_Mem();

  InitializeTouchPanel();
  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}