#include "Camera_Demo_objects.h"
#include "Camera_Demo_resources.h"

//--------------------- User code ---------------------//

#include "Camera_Demo.h"

static char bmpHeader[10] = {0x00,0x10,0xF0,0x00,0x40,0x01, 0xFF, 0xFF, 0xFF, 0xFF}; // Header of BMP in QVGA format.

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief Declarations of private functions */

static void TurnOffEffect(void);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   Inits Camera module.
 *
 * @param   none
 *
 * @return  none
 */

void InitCamera(void)
{
    Delay_ms(1000);
    strcpy(Button_Message_Caption, "Initialization of camera: START");
    DrawButton(&Button_Message);
    Delay_ms(1000);

    // Init Camera module
    if(Camera_InitQVGA() != _CAMERA_STATUS_OK)
    {
        strcpy(Button_Message_Caption, "Initialization of camera: FAIL");
        DrawButton(&Button_Message);
        while(1);                                                               // Initialization fails
    }

    strcpy(Button_Message_Caption, "Initialization of camera: COMPLETE");
    DrawButton(&Button_Message);
    Delay_ms(1000);
    TurnOffEffect();
    Delay_ms(500);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function is called from TFT_Get_Data driver function.
 *          On start of frame it return pointer to 320x240 bmp image header (QVGA format).
 *          Further, it return pointer to line pixels data.
 *
 * @param   bytesToGet - requested number of bytes.
 * @param   pBytesGot  - valid number of bytes in "return buffer"
 *
 * @return  none
 */

char *GetCameraFrame(unsigned long bytesToGet, unsigned long *pBytesGot)
{
    // Get header of BMP image?
    if(bytesToGet == 10)
    {
        *pBytesGot = sizeof(bmpHeader);
        Camera_WaitVSync();                                                     // Waits vertical synchronization signal (start of frame)
        return bmpHeader;
    }

    // Get BMP image pixels.
    else
    {
        *pBytesGot = _CAMERA_HOR_RES * 2;
        return Camera_GetFrameLine();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function simulates "display turn off" effect on welcome screen.
 *
 * @param   none
 *
 * @return  none
 */

static void TurnOffEffect(void)
{
    char cnt = 0;

    Line_Up.Visible = 1;
    Line_Down.Visible = 1;
    for(cnt = 0; cnt < 120; cnt++)
    {
        DrawLine(&Line_Up);
        Line_Up.First_Point_Y++;
        Line_Up.Second_Point_Y++;
        DrawLine(&Line_Down);
        Line_Down.First_Point_Y--;
        Line_Down.Second_Point_Y--;
        Delay_ms(1);
    }
    
    Delay_ms(100);
    
    Line_Up.Color = 0;
    Line_Up.Second_Point_X = 0;
    Line_Down.Color = 0;
    Line_Down.First_Point_X = 320;
    for(cnt = 0; cnt < 158; cnt++)
    {
        Line_Up.Second_Point_X++;
        DrawLine(&Line_Up);
        Line_Down.First_Point_X--;
        DrawLine(&Line_Down);
        Delay_ms(1);
    }
}

//----------------- End of User code ------------------//

// Event Handlers