/*
 * Project name:
     GLCD (Demonstration of the GLCD library routines)
 * Copyright:
     (c) mikroElektronika, 2015.
 * Revision History:
     20150202:
       - Initial release;
 * Description:
     This is a simple demonstration of the GLCD library routines:
     - Init and Clear (pattern fill)
     - Image display
     - Basic geometry - lines, circles, boxes and rectangles
     - Text display and handling
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       EasyFT90x v7
                      http://www.mikroe.com/easyft90x/
     Oscillator:      CPU at full system clock, 100.000MHz
     ext. modules:    ac:GLCD
     
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Turn on SW5.5
 */
 
//Declarations------------------------------------------------------------------
const code char truck_bmp[1024];
//--------------------------------------------------------------end-declarations

// Glcd module connections
unsigned char GLCD_DataPort at GPIO_PORT_32_39;

sbit GLCD_CS1           at GPIO_PIN46_bit;
sbit GLCD_CS2           at GPIO_PIN47_bit;
sbit GLCD_RS            at GPIO_PIN40_bit;
sbit GLCD_RW            at GPIO_PIN5_bit;
sbit GLCD_EN            at GPIO_PIN41_bit;
sbit GLCD_RST           at GPIO_PIN42_bit;
// End Glcd module connections

void delay2S(){                                  // 2 seconds delay function
  Delay_ms(2000);
}

void main() {
  char counter;
  char *someText;

  Glcd_Init();                                     // Initialize GLCD
  Glcd_Fill(0x00);                                 // Clear GLCD

  while(1) {
    Glcd_Image(truck_bmp);                         // Draw image
    Delay2s(); Delay2s();
    Glcd_fill(0x00);                               // Clear GLCD
    Glcd_PartialImage(0,0,68,30,128,64,truck_bmp); // Partial image
    Delay_ms(500);
    Glcd_PartialImage(24,16,68,30,128,64,truck_bmp);
    Delay_ms(500);
    Glcd_PartialImage(56,34,68,30,128,64,truck_bmp);
    Delay2s(); Delay2s();
    Glcd_Fill(0x00);                               // Clear GLCD

    Glcd_Box(62,40,124,56,1);                      // Draw box
    Glcd_Rectangle(5,5,84,35,1);                   // Draw rectangle
    Delay_ms(1000);
    Glcd_Rectangle_Round_Edges(2,2,87,38,7,1);
    Delay_ms(1000);
    Glcd_Rectangle_Round_Edges_Fill(8,8,81,32,12,1);
    Delay_ms(1000);
    Glcd_Line(0, 0, 127, 63, 1);                   // Draw line
    Delay2s();

    for(counter = 5; counter < 60; counter+=5 ){   // Draw horizontal and vertical lines
      Delay_ms(250);
      Glcd_V_Line(2, 54, counter, 1);
      Glcd_H_Line(2, 120, counter, 1);
    }
    Delay2s();

    Glcd_Fill(0x00);                             // Clear Glcd
    Glcd_Set_Font(Character8x7, 8, 7, 32);       // Choose font, see __Lib_GLCDFonts.c in Uses folder
    Glcd_Write_Text("mikroE", 5, 7, 2);          // Write string

    for (counter = 1; counter <= 10; counter++)  // Draw circles
      Glcd_Circle(63,32, 3*counter, 1);
    Delay2s();

    Glcd_Circle_Fill(63,32, 30, 1);              // Draw circles
    Delay2s();

    Glcd_Box(12,20, 70,57, 2);                   // Draw box
    Delay2s();

    Glcd_Fill(0xFF);                             // Fill Glcd

    Glcd_Set_Font(Font_Glcd_Character8x7, 8, 7, 32);  // Change font
    someText = "8x7 Font";
    Glcd_Write_Text(someText, 5, 0, 2);               // Write string
    Delay2s();

    Glcd_Set_Font(Font_Glcd_System3x5, 3, 5, 32);     // Change font
    someText = "3X5 CAPITALS ONLY";
    Glcd_Write_Text(someText, 60, 2, 2);              // Write string
    Delay2s();

    Glcd_Set_Font(Font_Glcd_5x7, 5, 7, 32);           // Change font
    someText = "5x7 Font";
    Glcd_Write_Text(someText, 5, 4, 2);               // Write string
    Delay2s();

    Glcd_Set_Font(Font_Glcd_System5x7, 5, 7, 32);     // Change font
    someText = "5x7 Font (v2)";
    Glcd_Write_Text(someText, 50, 6, 2);              // Write string
    Delay2s();
  }
}