/* Project name:
     TimerA_interrupt (TimerA interrupt test)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release;
 * Description:
     This code demonstrates how to use Timer A and it's interrupt.
     Program toggles LEDs on ports 00-07;
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       EasyFT90x v7
                      http://www.mikroe.com/easyft90x
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Turn ON ports GPIO 00-07 at SW8 (board specific).
 */


 void Timer() iv IVT_TIMERS_IRQ {
  // toggle PORT 00-07
  GPIO_PORT_00_07 = ~GPIO_PORT_00_07;
  // Clear Timer A interrupt flag
  TIMER_INT.B0 = 1;
}


void main() {
  GPIO_Digital_Output(&GPIO_PORT_00_07, _GPIO_PINMASK_ALL); // Set GPIO 00-07 port as digital output
  GPIO_PORT_00_07 = 0;
  // Enable the Timer module
  TIMER_CONTROL_0 = 2;
  // Select Timer A
  TIMER_SELECT = 0;
  // Write prescaler for Timer A
  TIMER_PRESC_LS = 10000;
  TIMER_PRESC_MS = 10000 >> 8;
  // Write start value
  TIMER_WRITE_LS = 10000;
  TIMER_WRITE_MS = 10000 >> 8;
  // Set continuous mode and direction down
  TIMER_CONTROL_3 = 0;
  // Trigger clearing timer A and prescaler
  TIMER_CONTROL_4 = 0x11;
  // Enable prescaler for Timer A
  TIMER_CONTROL_2 = 0x10;
  // Enable Timer A interrupt
  TIMER_INT.B1 = 1;
  // Enable global interrupts
  IRQ_CTRL.B31 = 0;
  // Start timer A
  TIMER_CONTROL_1 = 1;

  while(1);
}