/* Project name:
     WatchDog_Timer (Demonstration of the Watchdog Timer usage)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release;
 * Description:
     This code demonstrates how to use Watchdog timer.
     WDT doesn't reset the MCU automatically, instead, this is
     expected to be executed in the interrupt routine.
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       EasyFT90x v7
                      http://www.mikroe.com/easyft90x/

     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Pull down on GPIO 24-31 pins (board specific).
     - Upon entering the WD routine, core registers will be reset, while
       peripherial registers will retain their value.
       User should take this into account.
     - GPIO 24-31 port will swap its value on roughly every 0.5s
 */

void WDG_Interrupt() iv IVT_WDT_IRQ ics ICS_AUTO
{
  // toggle the port
  GPIO_PORT_24_31 = ~GPIO_PORT_24_31;
  Delay_ms(500);

  // reset peripherials
  PERI_SOFTRESET_bit = 1;
  while (PERI_SOFTRESET_bit == 1) {}
  
  // jump to the beginning
  asm {
    JMP 0
  }
}

void main()
{
  // enable digital output
  GPIO_Digital_Output(&GPIO_PORT_24_31, _GPIO_PINMASK_ALL);

  GPIO_PORT_24_31 = 0x33;
  Delay_ms(500);

  IRQ_CTRL.B31 = 0;

  // enable the timer
  BLOCK_EN_bit = 1;

  // WDT start value initialization:
  // only one bit can be set; setting another bit will clear it.
  TIMER_WDG = 0x16; // 2^22 * 10e-8 = 0.4194304s
  
  // clear the watchdog
  CLEAR_WDG_bit = 1;
  // enable WDG interrupt
  WDG_INT_IEN_bit = 1;
  // Enable the watchdog
  START_WDG_bit = 1;

  // endless loop; WDT routine will reset the MCU
  while (1) {}
}